<?php
/**
 * @copyright Copyright 2023 Genius Plugin Co., Ltd. © All Rights Reserved
 * @version 1.0.0
 * @author geniusplugin.com
 */
class GeniusPlugin_Module extends ServerModule
{
    public $api, $server, $_name;
    public $order = [], $product = [], $user = [];
    public $config = [], $options = [];
    public $lang, $error;
    public $area_link;
    public $domain_token;
    public $dashboard;
    public $requirements;
    function __construct($server, $options = [])
    {
        $this->_name = __CLASS__;
        parent::__construct($server, $options);
        $this->dashboard = 'https://hub.myplugin.online';
        // Add addons tweak to templates.
        $directory = $_SERVER['DOCUMENT_ROOT'] . '/templates/website/';
        $lineToAdd = '<?php include($_SERVER[\'DOCUMENT_ROOT\'] . \'/coremio/modules/Servers/GeniusPlugin/pages/addon.php\'); ?>';
        $this->traverseAndAddLine($directory, $lineToAdd); 
    }
    public function define_server_info($server = [])
    {
        if (!class_exists("GeniusPlugin_Api")) include __DIR__ . DS . "api.php";
        $this->api = new GeniusPlugin_Api($server["name"], $server["username"], $server["password"], $server["access_hash"]);
    }
    /**
     * Method to test connection and add addons tweak to templates.
     *
     * @return string|bool Returns 'OK' if the connection is successful, otherwise returns an error message.
     */
    public function testConnect()
    {
        
        // Check remote server connection.
        try {
            $connect    = $this->api->testConnection();
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            return false;
        }
        if ($connect != 'OK') {
            $this->error = $connect;
            return false;
        }
        return $connect; //true;
    }
    public function config_options($data = [])
    {
        return [
            'token'          => [
                'name'              => "Token",
                'description'       => "Firewall License Token",
                'type'              => "text",
                'width'             => "42",
                'value'             => isset($data["token"]) ? $data["token"] : "",
                'placeholder'       => "",
            ],
        ];
    }
    public function clientArea()
    {
        $content    = '';
        $_page      = $this->page;
        $_data      = [];
        if (!$_page) $_page = 'home';
        $token = $this->options["config"]['token'];
        $domain_password = $this->options["config"]["password"];
        $domain_name = $this->options["config"]["user"];
        $hub = $this->api->getTowerRs($domain_name);
        if (is_array($hub) && filter_var('https://' . $hub[1], FILTER_VALIDATE_URL) !== false) {
            $this->dashboard = 'https://' . $hub[1];
        }
        if ($_page == "home") {
            $_data = ['token' => $token, 'domain_password' => $domain_password, 'domain_name' => $domain_name, 'dashboard' => $this->dashboard];
        }
        $content .= $this->clientArea_buttons_output();
        $content .= $this->get_page('clientArea-' . $_page, $_data);
        return  $content;
    }
    public function clientArea_buttons()
    {
        if ($this->page && $this->page != "home") {
            $buttons['home'] = [
                'text' => $this->lang["turn-back"],
                'type' => 'page-loader',
            ];
        } else {
            $buttons['change-password'] = [
                'text'  => 'Change Password',
                'type'  => 'page-loader',
            ];
            $buttons['custom_function'] = [
                'text'  => 'Panel',
                'type'  => 'link',
                'url'   => $this->dashboard,
                'target_blank' => true,
            ];
            $buttons['domain_name'] = [
                'text'      => $this->requirements[0]['response'],
                'type'      => 'link',
                'url'       => 'https://' . $this->requirements[0]['response'],
                'target_blank' => true,
            ];
        }
        return $buttons;
    }
    public function use_clientArea_SingleSignOn()
    {
        $domain_name = $this->requirements[0]['response'];
        $domain_password = $this->options["config"]["password"];
        if (empty($domain_name || empty($domain_password))) {
            echo "Invalid domain or password, unable to access.";
            return false;
        }
        $api_result     = $this->api->getDirectLink($domain_name, $domain_password, false);
        if (substr($api_result, 0, 2) != 'OK') {
            echo "An error has occurred, unable to access.";
            return false;
        }
        $token          = substr($api_result, 2);
        Utility::redirect($token);
        echo "Please allow popup from your client area. Redirecting...";
    }
    public function use_adminArea_SingleSignOn()
    {
        $domain_name = $this->requirements[0]['response'];
        $domain_password = $this->options["config"]["password"];
        if (empty($domain_name || empty($domain_password))) {
            echo "Invalid domain or password, unable to access.";
            return false;
        }
        $api_result     = $this->api->getDirectLink($domain_name, $domain_password, false);
        if (substr($api_result, 0, 2) != 'OK') {
            echo "An error has occurred, unable to access.";
            return false;
        }
        $token          = substr($api_result, 2);
        Utility::redirect($token);
        echo "Please allow popup from your client area. Redirecting...";
    }
    /**
     * Method to retrieve the status of the package.
     *
     * @return bool Returns true if the status retrieval is successful, otherwise returns false with an error message.
     */
    public function get_status()
    {
        $domain_name = $this->requirements[0]['response'];
        try {
            $request = $this->api->Status($domain_name);
            $result = json_decode($request, true);
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $e->getMessage(), $e->getTraceAsString());
            return false;
        }
        if ($result['status'] == 'successful') {
            return true;
        } else {
            $this->error = $result;
            return false;
        }
    }
    public function use_adminArea_Status()
    {
        $domain_name = $this->requirements[0]['response'];
        try {
            $request = $this->api->Status($domain_name);
            $result = json_decode($request, true);
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $e->getMessage(), $e->getTraceAsString());
            echo Utility::jencode([
                'status' => "error",
                'message' => $result['message'],
            ]);
            return false;
        }
        if ($result['status'] == 'successful') {
            echo Utility::jencode([
                'status' => "successful",
                'message' => $result['message']
            ]);
            return true;
        } else {
            $this->error = $result;
            echo Utility::jencode([
                'status' => "error",
                'message' => $result['message']
            ]);
            return false;
        }
    }
    /**
     * Method to check credit using the admin area API.
     *
     * @return bool Returns true after checking credit using the admin area API.
     */
    public function use_adminArea_CheckCredit()
    {
        $response = $this->api->CheckCredit();
        if (is_object($response) && substr($response->status, 0, 2) !== 'OK') {
            echo json_encode(array(
                'status' => "error",
                'message' =>  $response->status
            ));
            return false;
        } elseif (is_object($response) && substr($response->status, 0, 2) === 'OK') {
            echo json_encode(array(
                'status' => "successful",
                'message' =>  $response->status
            ));
            return true;
        } else {
            echo json_encode(array(
                'status' => "error",
                'message' =>  $response
            ));
            return false;
        }
    }
    function findPackage($inputString)
    {
        $trimmedInput = trim(strtolower($inputString));
        $validPackages = array('Free', '20k', '40k', '80k', '200k', '500k');
        if (in_array($trimmedInput, array_map('strtolower', $validPackages))) {
            return $validPackages[array_search($trimmedInput, array_map('strtolower', $validPackages))];
        }
        if (preg_match('/(?:\bfirewall\b.*?\b|\b)\K(free|20k|40k|80k|200k|500k)\b/i', $trimmedInput, $matches)) {
            return $validPackages[array_search(strtolower($matches[1]), array_map('strtolower', $validPackages))];
        }

        return null; // Return null if no match
    }
    public function create()
    {
        $domain_password       = Utility::generate_hash(8);
        $password = $this->encode_str($domain_password);
        $domain_name = $this->requirements[0]['response'];
        $domain_email = $this->user["email"];
        $domain_package = $this->findPackage($this->product["title"]);
        if (empty($domain_name)) {
            return false;
        } elseif (empty($domain_password)) {
            return false;
        } elseif (empty($domain_email)) {
            return false;
        } elseif (empty($domain_package)) {
            return false;
        } else {
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], json_encode(array($domain_name, $domain_password, $domain_email, $domain_package)) . json_encode($this), '');
            $request = $this->api->Create($domain_name, $domain_password, $domain_email, $domain_package);
        }
        $result = json_decode($request, true);
        if (is_array($result) && $result['status'] === 'successful') {
            $this->options["config"]['token'] = $result['token'];
            $this->options["config"]["password"] = $password;
            $this->options["config"]["user"] = $domain_name;
            $this->options['ftp_info']['username'] =  $this->requirements[0]['response'];
            $this->options['ftp_info']['password'] =  $password;
            $this->options["login"]["password"] = $password;
            $this->options['domain'] =  $this->requirements[0]['response'];
            Orders::set($this->order["id"], ['options' => Utility::jencode($this->options)]);
            Orders::set($this->order["id"], ['status' => 'active']);
            return false;
        }
        $this->error = $result;
        self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $result, '');
        return false;
    }
    public function use_adminArea_Create()
    {
        $domain_password       = Utility::generate_hash(8);
        $password = $this->encode_str($domain_password);
        $domain_name = $this->requirements[0]['response'];
        $domain_email = $this->user["email"];
        $domain_package = $this->findPackage($this->product["title"]);
        if (empty($domain_password)) {
            echo Utility::jencode([
                'status' => "error",
                'message' => 'Password for the new account is invalid: ' . $domain_password,
            ]);
            return false;
        } elseif (empty($domain_email)) {
            echo Utility::jencode([
                'status' => "error",
                'message' => 'User email is invalid: ' . $domain_email,
            ]);
            return false;
        } elseif (empty($domain_package)) {
            echo Utility::jencode([
                'status' => "error",
                'message' => 'Package is invalid: ' . $domain_package,
            ]);
            return false;
        } else {
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], json_encode(array($domain_name, $domain_password, $domain_email, $domain_package)) . json_encode($this), '');
            $request = $this->api->Create($domain_name, $domain_password, $domain_email, $domain_package);
        }
        $result = json_decode($request, true);
        if (is_array($result) && $result['status'] === 'successful') {
            $this->options["config"]['token'] = $result['token'];
            $this->options["config"]["password"] = $password;
            $this->options["config"]["user"] = $domain_name;
            $this->options['domain'] =  $this->requirements[0]['response'];
            $this->options['ftp_info']['username'] =  $this->requirements[0]['response'];
            $this->options['ftp_info']['password'] =  $password;
            $this->options["login"]["password"] = $password;
            Orders::set($this->order["id"], ['options' => Utility::jencode($this->options)]);
            Orders::set($this->order["id"], ['status' => 'active']);
            echo json_encode([
                'status' => "successful",
                'message' => $result['token'],
                'timeRedirect' => ['url' => $this->area_link, 'duration' => 3000],
            ]);
            return true;
        }
        self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $result, '');
        if (is_array($result)) {
            $message = $result['message'];
        } elseif (is_object($result)) {
            $message = $result->message;
        } else {
            $message = $result;
        }
        $this->error = $result;
        echo Utility::jencode([
            'status' => "error",
            'message' => $message,
        ]);
        return false;
    }
    public function suspend()
    {
        $domain_name = $this->requirements[0]['response'];
        if (empty($domain_name)) {
            return false;
        }
        try {
            $request = $this->api->Suspend($domain_name);
            $result = json_decode($request, true);
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $e->getMessage(), $e->getTraceAsString());
            return false;
        }
        if ($result['status'] == 'successful') {
            Orders::set($this->order["id"], ['status' => 'suspended']);
            return true;
        } else {
            return false;
        }
    }
    public function use_adminArea_Suspend()
    {
        $domain_name = $this->requirements[0]['response'];
        if (empty($domain_name)) {
            echo Utility::jencode([
                'status' => "error",
                'message' => 'Domain name is invalid: ' . $domain_name,
            ]);
            return false;
        }
        try {
            $request = $this->api->Suspend($domain_name);
            $result = json_decode($request, true);
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $e->getMessage(), $e->getTraceAsString());
            echo Utility::jencode([
                'status' => "error",
                'message' => $result['message'],
            ]);
            return false;
        }
        if ($result['status'] == 'successful') {
            Orders::set($this->order["id"], ['status' => 'suspended']);
            echo Utility::jencode([
                'status' => "successful",
                'message' => $result['message'],
                'timeRedirect' => ['url' => $this->area_link, 'duration' => 3000],
            ]);
            return true;
        } else {
            $this->error = $result;
            echo Utility::jencode([
                'status' => "error",
                'message' => $result['message']
            ]);
            return false;
        }
    }
    public function unsuspend()
    {
        $domain_name = $this->requirements[0]['response'];
        if (empty($domain_name)) {
            return false;
        }
        try {
            $request = $this->api->Unsuspend($domain_name);
            $result = json_decode($request, true);
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $e->getMessage(), $e->getTraceAsString());
            return false;
        }
        if ($result['status'] == 'successful') {
            Orders::set($this->order["id"], ['status' => 'active']);
            return true;
        } else {
            $this->error = $result;
            return false;
        }
    }
    public function use_adminArea_Unsuspend()
    {
        $domain_name = $this->requirements[0]['response'];
        if (empty($domain_name)) {
            echo Utility::jencode([
                'status' => "error",
                'message' => 'Domain name is invalid: ' . $domain_name,
            ]);
            return false;
        }
        try {
            $request = $this->api->Unsuspend($domain_name);
            $result = json_decode($request, true);
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            echo Utility::jencode([
                'status' => "error",
                'message' => $result['message'],
            ]);
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $e->getMessage(), $e->getTraceAsString());
            return false;
        }
        if ($result['status'] == 'successful') {
            Orders::set($this->order["id"], ['status' => 'active']);
            echo Utility::jencode([
                'status' => "successful",
                'message' => $result['message'],
                'timeRedirect' => ['url' => $this->area_link, 'duration' => 3000],
            ]);
            return true;
        } else {
            $this->error = $result;
            echo Utility::jencode([
                'status' => "error",
                'message' => $result['message']
            ]);
            return false;
        }
    }
    public function use_adminArea_Renew()
    {
        $domain_name = $this->requirements[0]['response'];
        if (empty($domain_name)) {
            echo Utility::jencode([
                'status' => "error",
                'message' => 'Domain name is invalid: ' . $domain_name,
            ]);
            return false;
        }
        try {
            $request = $this->api->Renew($domain_name);
            $result = json_decode($request, true);
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $e->getMessage(), $e->getTraceAsString());
            echo Utility::jencode([
                'status' => "error",
                'message' => $result['message'],
            ]);
            return false;
        }
        if ($result['status'] == 'successful') {
            Orders::set($this->order["id"], ['status' => 'active']);
            echo Utility::jencode([
                'status' => "successful",
                'message' => $result['message'],
                'timeRedirect' => ['url' => $this->area_link, 'duration' => 3000],
            ]);
            return true;
        } else {
            $this->error = $result;
            echo Utility::jencode([
                'status' => "error",
                'message' => $result['message']
            ]);
            return false;
        }
    }
    public function delete()
    {
        $domain_name = $this->requirements[0]['response'];
        if (empty($domain_name)) {
            return false;
        }
        try {
            $request = $this->api->Delete($domain_name);
            $result = json_decode($request, true);
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $e->getMessage(), $e->getTraceAsString());
            return false;
        }
        if ($result['status'] == 'successful') {
            Orders::set($this->order["id"], ['status' => 'cancelled']);
            return true;
        } else {
            $this->error = $result;
            return false;
        }
    }
    public function use_adminArea_Delete()
    {
        $domain_name = $this->requirements[0]['response'];
        if (empty($domain_name)) {
            echo Utility::jencode([
                'status' => "error",
                'message' => 'Domain name is invalid: ' . $domain_name,
            ]);
            return false;
        }
        try {
            $request = $this->api->Delete($domain_name);
            $result = json_decode($request, true);
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $e->getMessage(), $e->getTraceAsString());
            echo Utility::jencode([
                'status' => "error",
                'message' => $result['message'],
            ]);
            return false;
        }
        if ($result['status'] == 'successful') {
            Orders::set($this->order["id"], ['status' => 'cancelled']);
            echo Utility::jencode([
                'status' => "successful",
                'message' => $result['message'],
                'timeRedirect' => ['url' => $this->area_link, 'duration' => 3000],
            ]);
            return true;
        } else {
            $this->error = $result;
            echo Utility::jencode([
                'status' => "error",
                'message' => $result['message']
            ]);
            return false;
        }
    }
    public function reinstall()
    {
        return false;
    }
    public function terminate()
    {
        $domain_name = $this->requirements[0]['response'];
        if (empty($domain_name)) {
            return false;
        }
        try {
            $request = $this->api->Delete($domain_name);
            $result = json_decode($request, true);
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $e->getMessage(), $e->getTraceAsString());
            return false;
        }
        if ($result['status'] == 'successful') {
            Orders::set($this->order["id"], ['status' => 'cancelled']);
            return true;
        } else {
            $this->error = $result;
            return false;
        }
    }
    public function renew()
    {
        $domain_name = $this->requirements[0]['response'];
        if (empty($domain_name)) {
            return false;
        }
        try {
            $request = $this->api->Renew($domain_name);
            $result = json_decode($request, true);
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $e->getMessage(), $e->getTraceAsString());
            return false;
        }
        if ($result['status'] == 'successful') {
            Orders::set($this->order["id"], ['status' => 'active']);
            return true;
        } else {
            $this->error = $result;
            return false;
        }
    }
    public function get_password()
    {
        return $this->decode_str($this->options["config"]["password"]);
    }
    public function apply_updowngrade()
    {
        $domain_package = $this->order['name']; //$this->product["title"];
        $query = WDB::select("title")
            ->from("users_products_updowngrades")
            ->join("INNER", "products_lang", "`users_products_updowngrades`.`new_pid` = `products_lang`.`owner_id`")
            ->where("`users_products_updowngrades`.`owner_id`", "=", $this->order['id'])
            ->order_by("users_products_updowngrades.id DESC")
            ->limit(1)
            ->build(true)
            ->fetch_assoc();
        if (is_array($query) && count($query) === 1) {
            $domain_package = $this->findPackage($query[0]['title']);
        }else{
            $domain_package = $this->findPackage($domain_package);
        }
        if($this->get_status() === false){
            $this->error = 'Domain not active';
            self::save_log('Servers', $this->_name, 'get_status', ['order' => $this->order], $this->error, '');
            return false;
        }  
        $domain_name = $this->requirements[0]['response'];
        $domain_status =  $this->product['status'];
        if (empty($domain_name) || empty($domain_package) || empty($domain_status) || $domain_status != 'active') {
            $this->error = 'Missing data';
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $this->error, '');
            return false;
        }
        try {
            $request = $this->api->Update($domain_name, $domain_package, $domain_status);
            $result = json_decode($request, true);
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $e->getMessage(), $e->getTraceAsString());
            return false;
        }
        self::save_log('Servers', $this->_name, __FUNCTION__ . '2', $result['token'], $this->options["config"]['token'], $this->options);
        if ($result['status'] == 'successful') {
            //Orders::set($this->order["id"], ['status' => 'active']);
            return true;
        } else {
            return false;
        }
    }
    public function use_adminArea_Update()
    {
        if ($this->get_status() === false) {
            echo Utility::jencode([
                'status' => "error",
                'message' => 'Service is not active',
            ]);
            return false;
        }    
        $domain_name = $this->requirements[0]['response'];
        $domain_package = $this->findPackage($this->order['name']); //$this->product["title"];
        $domain_status =  $this->product['status'];
        if (empty($domain_name)) {
            echo Utility::jencode([
                'status' => "error",
                'message' => 'Domain name is invalid: ' . $domain_name,
            ]);
            return false;
        }
        if (empty($domain_package)) {
            echo Utility::jencode([
                'status' => "error",
                'message' => 'Package is invalid: ' . $domain_package,
            ]);
            return false;
        }
        try {
            $request = $this->api->Update($domain_name, $domain_package, $domain_status);
            $result = json_decode($request, true);
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], $e->getMessage(), $e->getTraceAsString());
            echo Utility::jencode([
                'status' => "error",
                'message' => $result['message'],
            ]);
            return false;
        }
        if (empty($this->options["config"]["password"]) && !empty($this->options['ftp_info']['password'])) {
            $this->options["config"]["password"] = $this->encode_str($this->options['ftp_info']['password']);
        }
        if (empty($this->options["config"]["password"])) {
            $password       = Utility::generate_hash(8);
            $password_request = $this->api->ResetPass($this->requirements[0]['response'], $password);
            $password_request = json_decode($password_request);
            if (!is_object($password_request) || $password_request->status !== 'successful') {
                echo  Utility::jencode([
                    'status' => "error",
                    'message' => 'Error changing password. Try again later or contact support ',
                ]);
                $this->error = $password_request;
                return false;
            }
            $this->options["config"]["password"] = $this->encode_str($password);
        } else {
            $this->options["config"]["password"] = $this->encode_str($this->options["config"]["password"]);
        }
        // Any option not set, would be overwriten with empty value.
        $this->options["login"]["password"] = $this->options["config"]["password"];
        $this->options['ftp_info']['password'] =  $this->options["config"]["password"];
        $this->options["config"]["user"] = $this->requirements[0]['response'];
        $this->options['ftp_info']['username'] =  $this->requirements[0]['response'];
        $this->options['domain'] =  $this->requirements[0]['response'];
        if (isset($result['token']) && !empty($result['token'])) {
            $this->options["config"]['token'] = $result['token'];
        }
        $this->options["config"]['token'] = $this->options["config"]['token'];
        Orders::set($this->order["id"], ['options' => Utility::jencode($this->options)]);
        if ($result['status'] == 'successful') {
            echo Utility::jencode([
                'status' => "successful",
                'message' => $result['message'],
                'timeRedirect' => ['url' => $this->area_link, 'duration' => 3000],
            ]);
            return true;
        } else {
            echo Utility::jencode([
                'status' => "error",
                'message' => 'Please check the package data. ' . $result['message'],
                'timeRedirect' => ['url' => $this->area_link, 'duration' => 3000],
            ]);
            $this->error = $result;
            return false;
        }
    }
    public function use_clientArea_changePassword()
    {
        if (Filter::POST("var1")) {
            $password = Filter::POST("var1");
        } else {
            echo  Utility::jencode([
                'status' => "error",
                'message' => "Invalid Password",
            ]);
            return false;
        }
        if (strlen($password) < 8) {
            echo  Utility::jencode([
                'status' => "error",
                'message' => "Password must be 8-24 characters. Must contains 1 uppercase, 1 lowercase, 1 digit and 1 special character $@$!#..etc",
            ]);
            return false;
        }
        $password_e         = $this->encode_str($password);
        $result = $this->api->ResetPass($this->requirements[0]['response'], $password);
        $result = json_decode($result);
        if (is_object($result) && $result->status !== 'successful') {
            echo  Utility::jencode([
                'status' => "error",
                'message' => 'Error changing password. Try again later or contact support ',
            ]);
            $this->error = $result;
            return false;
        } else {
            if (!isset($this->options["login"])) $this->options["login"] = [];
            $this->options["login"]["password"] = $password_e;
            $this->options["config"]["password"] = $password_e;
            $this->options['ftp_info']['password'] =  $password_e;
            Orders::set($this->order["id"], ['options' => Utility::jencode($this->options)]);
            // Save Action Log
            $u_data     = UserManager::LoginData("member");
            $user_id    = $u_data["id"];
            User::addAction($user_id, 'transaction', 'The server password for service #' . $this->order["id"] . ' has been changed.');
            Orders::add_history($user_id, $this->order["id"], 'server-order-password-changed');
            // Save Module Log
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], ['api_result' => $result]);
            echo Utility::jencode([
                'status'    => "successful",
                'message'   => $this->lang["successful"],
                'timeRedirect' => ['url' => $this->area_link, 'duration' => 3000],
            ]);
            return true;
        }
        return false;
    }
    public function use_adminArea_ResetPassword()
    {
        $password       = Utility::generate_hash(8);
        $password_e = $this->encode_str($password);
        $result = $this->api->ResetPass($this->requirements[0]['response'], $password);
        $result = json_decode($result);
        if (is_object($result) && $result->status !== 'successful') {
            $this->error = $result;
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], ['api_result' => $result]);
            echo  Utility::jencode([
                'status' => "error",
                'message' => 'Error changing password. Try again later or contact support ',
            ]);
            return false;
        } else {
            $this->options["login"]["password"] = $password_e;
            $this->options["config"]["password"] = $password_e;
            $this->options['ftp_info']['password'] =  $password_e;
            Orders::set($this->order["id"], ['options' => Utility::jencode($this->options)]);
            // Save Action Log
            $u_data     = UserManager::LoginData("member");
            $user_id    = $u_data["id"];
            User::addAction($user_id, 'transaction', 'The server password for service #' . $this->order["id"] . ' has been changed.');
            Orders::add_history($user_id, $this->order["id"], 'server-order-password-changed');
            // Save Module Log
            self::save_log('Servers', $this->_name, __FUNCTION__, ['order' => $this->order], ['api_result' => $result]);
            echo Utility::jencode([
                'status'    => "successful",
                'message'   => $this->lang["successful"],
                'timeRedirect' => ['url' => $this->area_link, 'duration' => 1000],
            ]);
            return true;
        }
        return false;
    }
    public function adminArea_buttons()
    {
        $this->get_status();
        $buttons = [];
        $status =  $this->api->getAvailableActions($this->requirements[0]['response']);
        if (is_array($status)) {
            foreach ($status as $available_service) {
                $buttons[$available_service] = [
                    'text'  => $available_service,
                    'type'  => 'transaction',
                ];
            }
        }
        if (in_array('Renew', $status)) {
            $buttons['ResetPassword'] = [
                'text'  => 'Change Password',
                'type'  => 'transaction',
            ];
        }
        return $buttons;
    }
    /**
     * Recursively traverse a directory, and add a line to PHP files with specific filenames to handle the addon group feature
     *
     * @param string $directory The base directory to start traversal.
     * @param string $lineToAdd The line to add to PHP files.
     */
    private function traverseAndAddLine($directory, $lineToAdd)
    {
        $files = scandir($directory);
        foreach ($files as $file) {
            if ($file !== '.' && $file !== '..') {
                $path = $directory . '/' . $file;
                if (is_dir($path)) {
                    $this->traverseAndAddLine($path, $lineToAdd);
                } elseif (is_file($path) && pathinfo($path, PATHINFO_EXTENSION) === 'php') {
                    $fileContent = file_get_contents($path);
                    $filename = pathinfo($path, PATHINFO_FILENAME);
                    if ($filename === 'order-steps-server' || $filename === 'order-steps-hosting') {
                        if (strpos($fileContent, $lineToAdd) === false) {
                            file_put_contents($path, $fileContent . "\n" . $lineToAdd . "\n");
                        }
                    }
                }
            }
        }
    }
}
