<?php
/**
 * @copyright Copyright 2023 Genius Plugin, Corp. © All Rights Reserved
 * @version 1.0.0
 * @author geniusplugin.com
 */

class GeniusPlugin_Api
{

    private $server;
    private $apikey;
    public $_name;
    public $error;

    public function __construct($server, $rs_username,$rs_password, $access_hash)
    {
        $this->_name = __CLASS__;
        $this->server = $server;
        $this->apikey = $access_hash;
    }

    public function testConnection()
    {
        $apikey = sha1(trim($this->apikey));
        $timestamp = str_shuffle(time());
        $domain = 'example.com';
        $hash = sha1($apikey . $domain . $timestamp);
        $endpoint = 'connection';
        $data = [
            'domain' => $domain,
            'hash' => $hash,
            'timestamp' => $timestamp

        ];
        if ($data !== array_filter($data, 'strlen') || empty($this->apikey) || empty($this->server)) {
            return json_encode(array(
                'status' => "error",
                'message' =>  'Request missing data'
            ));
        }
        $response = json_decode($this->call($endpoint,  'POST', $data));
        if (!is_object($response)) {
            if (substr($response, 0, 2) != 'OK') {
                return $response;
            } else {
                return 'OK';
            }
        } else {
            if (substr($response->status, 0, 2) != 'OK') {
                return $response->status;
            } else {
                return 'OK';
            }
        }
    }

    public function call($endpoint,  $method = 'GET', $data = [])
    {
        $data = http_build_query($data);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://' . $this->server  . '/api/v1/' . $endpoint . '/');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_MAXREDIRS, 3);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);
        curl_setopt($ch, CURLOPT_TIMEOUT, 20);

        switch ($method) {
            case 'GET':
                break;
            case 'POST':
                curl_setopt($ch, CURLOPT_POST, true);
                curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
                break;
            case 'PUT':
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');
                curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
                break;
            case 'DELETE':
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'DELETE');
                break;
        }

        $response = curl_exec($ch);
        curl_close($ch);
        if (curl_errno($ch)) {
            $this->error = curl_errno($ch);
            Modules::save_log("Servers", __CLASS__, $endpoint, $data, $response, $this->error ? $this->error : $response);
            return json_encode([
                'status' => "error",
                'message' =>  curl_errno($ch)
            ]);
        }

        Modules::save_log("Servers", __CLASS__, $endpoint, $data, $response, $this->error ? $this->error : '');
        return $response;
    }

    /**
     * Function used to encode the user password
     * We encourage you to strengthen the password. GPF can refuse weaker
     * @param [string] $apikey = your apikey
     * @param [string] $string = user password
     * @return string
     */
    public function gpfPassEncrypt($apikey, $string)
    {
        if (empty($apikey) || empty($string)) return null;
        $ivlen = openssl_cipher_iv_length($cipher = "AES-128-CBC");
        $iv = openssl_random_pseudo_bytes($ivlen);
        $ciphertext_raw = openssl_encrypt($string, $cipher, $apikey, OPENSSL_RAW_DATA, $iv);
        $hmac = hash_hmac('sha256', $ciphertext_raw, $apikey, $as_binary = true);
        $ciphertext = base64_encode($iv . $hmac . $ciphertext_raw);
        return $ciphertext;
    }

    public function CheckCredit()
    {

        $apikey = sha1(trim($this->apikey));
        $timestamp = str_shuffle(time());
        $domain = 'example.com';
        $hash = sha1($apikey . $domain . $timestamp);
        $endpoint = 'getcredit';
        $data = [
            'domain' => $domain,
            'hash' => $hash,
            'timestamp' => $timestamp

        ];
        if ($data !== array_filter($data, 'strlen') || empty($this->apikey) || empty($this->server)) {
            return json_encode(array(
                'status' => "error",
                'message' =>  'Request missing data'
            ));
        }
        return json_decode($this->call($endpoint, 'POST', $data));
    }

    public function Status($domain_name)
    {
        if ($domain_name == '') {
            return json_encode(array(
                'status' => "error",
                'message' =>  'invalid'
            ));
        }

        $apikey = sha1(trim($this->apikey));
        $endpoint = 'list';
        $timestamp = str_shuffle(time());
        $hash = sha1($apikey .  $domain_name . $timestamp);
        $data = [
            'domain' => $domain_name,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];
        if ($data !== array_filter($data, 'strlen') || empty($this->apikey) || empty($this->server)) {
            return json_encode(array(
                'status' => "error",
                'message' =>  'Request missing data'
            ));
        }
        $response = json_decode($this->call($endpoint, 'POST', $data));

        if (!isset($response->exist) || $response->exist != 'exists') {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response->status
            ));
        } else {
            if ($response->status !== 'active') {
                return json_encode(array(
                    'status' => "error",
                    'message' =>  $response->status
                ));
            } else {
                return json_encode(array(
                    'status' => "successful",
                    'message' =>  $response->status,
                    'token' => $response->token
                ));
            }
        }
        return json_encode(array(
            'status' => "error",
            'message' =>  'unknown'
        ));
    }

    public function getAvailableActions($domain_name)
    {
        $actions = array();

        if ($domain_name == '') {
            $actions[] = 'CheckCredit';
            $actions[] = 'Create';
            $actions[] = 'Status';
            return $actions;
        }

        $apikey = sha1(trim($this->apikey));
        $endpoint = 'list';
        $timestamp = str_shuffle(time());
        $hash = sha1($apikey .  $domain_name . $timestamp);
        $data = [
            'domain' => $domain_name,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];
        if ($data !== array_filter($data, 'strlen') || empty($this->apikey) || empty($this->server)) {
            return json_encode(array(
                'status' => "error",
                'message' =>  'Request missing data'
            ));
        }
        $response = json_decode($this->call($endpoint, 'POST', $data));
        $actions[] = 'Status';
        $actions[] = 'CheckCredit';
        if (!isset($response->exist) || $response->exist != 'exists') {
            $actions[] = 'Create';
        } else {
            if ($response->status == 'suspended') {
                $actions[] = 'UnSuspend';
            } else {
                $actions[] = 'Suspend';
                $actions[] = 'Update';
                $actions[] = 'Renew';
            }
            $actions[] = 'Delete';
        }

        return $actions;
    }

    public function Create($domain_name, $domain_password, $domain_email, $domain_package)
    {
        $apikey = sha1(trim($this->apikey));
        $timestamp = str_shuffle(time());
        $hash = sha1($apikey . $domain_name . $timestamp);
        $password = $this->gpfPassEncrypt($this->apikey, $domain_password);
        $endpoint = 'autoregister';
        $data = [
            'package' => $domain_package,
            'password' => $password,
            'domain' => $domain_name,
            'uemail' => $domain_email,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];
        if ($data !== array_filter($data, 'strlen') || empty($this->apikey) || empty($this->server)) {
            return json_encode(array(
                'status' => "error",
                'message' =>  'Request missing data'
            ));
        }
        $response = json_decode($this->call($endpoint,  'POST', $data));
        if (is_object($response) && substr($response->status, 0, 2) != 'OK') {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response->status
            ));
        } elseif (is_object($response) && substr($response->status, 0, 2) == 'OK') {
            return json_encode(array(
                'status' => "successful",
                'token' =>  $response->token
            ));
        } else {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response
            ));
        }
    }

    public function Suspend($domain_name)
    {

        $apikey = sha1(trim($this->apikey));
        $endpoint = 'suspended';
        $timestamp = str_shuffle(time());
        $hash = sha1($apikey . $domain_name . $timestamp);
        $data = [
            'domain' => $domain_name,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];
        if ($data !== array_filter($data, 'strlen') || empty($this->apikey) || empty($this->server)) {
            return json_encode(array(
                'status' => "error",
                'message' =>  'Request missing data'
            ));
        }
        $response = json_decode($this->call($endpoint,  'POST', $data));
        if (is_object($response) && substr($response->status, 0, 2) != 'OK') {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response->status
            ));
        } elseif (is_object($response) && substr($response->status, 0, 2) == 'OK') {
            return json_encode(array(
                'status' => "successful",
                'message' =>  $response->status
            ));
        } else {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response
            ));
        }
    }

    public function Renew($domain_name)
    {
        $apikey = sha1(trim($this->apikey));
        $endpoint = 'renew';
        $timestamp = str_shuffle(time());
        $hash = sha1($apikey . $domain_name . $timestamp);
        $data = [
            'domain' => $domain_name,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];
        if ($data !== array_filter($data, 'strlen') || empty($this->apikey) || empty($this->server)) {
            return json_encode(array(
                'status' => "error",
                'message' =>  'Request missing data'
            ));
        }
        $response = json_decode($this->call($endpoint,  'POST', $data));
        if (is_object($response) && substr($response->status, 0, 2) != 'OK') {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response->status
            ));
        } elseif (is_object($response) && substr($response->status, 0, 2) == 'OK') {
            return json_encode(array(
                'status' => "successful",
                'message' =>  $response->status
            ));
        } else {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response
            ));
        }
    }

    public function Delete($domain_name)
    {
        $apikey = sha1(trim($this->apikey));
        $endpoint = 'delete';
        $timestamp = str_shuffle(time());
        $hash = sha1($apikey . $domain_name . $timestamp);
        $data = [
            'domain' => $domain_name,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];
        if ($data !== array_filter($data, 'strlen') || empty($this->apikey) || empty($this->server)) {
            return json_encode(array(
                'status' => "error",
                'message' =>  'Request missing data'
            ));
        }
        $response = json_decode($this->call($endpoint,  'POST', $data));
        if (is_object($response) && substr($response->status, 0, 2) != 'OK') {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response->status
            ));
        } elseif (is_object($response) && substr($response->status, 0, 2) == 'OK') {
            return json_encode(array(
                'status' => "successful",
                'message' =>  $response->status
            ));
        } else {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response
            ));
        }
    }

    /**
     * Get hub server parameters.
     * @param array $params common module parameters
     * @return array
     */
    public function getTowerRs($domain_name)
    {
        $timestamp = str_shuffle(time());
        $apikey = sha1(trim($this->apikey));
        $hash = sha1($apikey . $domain_name . $timestamp);
        $ch =  curl_init('https://hub.geniusplugin.com/license.php?v=1.0.0&app=ce&action=rs_hub&domain=' . $domain_name . '&apikey=' . $hash . '&timestamp=' . $timestamp);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_MAXREDIRS, 3);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);
        curl_setopt($ch, CURLOPT_TIMEOUT, 5);
        curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        $error = '';
        $result = curl_exec($ch);
        if (curl_errno($ch)) {
            $error = curl_error($ch);
            Modules::save_log("Servers", __CLASS__, __FUNCTION__, $domain_name, $result, $error);
        } 
        curl_close($ch);
        if (!empty($result)) {
            $server = json_decode($result);
            if (is_array($server)) {
                return $server;
            }
        }
       
    }

    public function getDirectLink($domain_name, $domain_password)
    {
        $apikey = sha1(trim($this->apikey));
        $timestamp = str_shuffle(time());
        $hash = sha1($apikey .  $domain_name . $timestamp);
        $hub = $this->getTowerRs($hash, $domain_name, $timestamp);

        $hub = $hub[1];
        if (empty($hub)) {
            $hub = 'https://hub.myplugin.online';
        };

        $password = $this->gpfPassEncrypt($this->apikey, $domain_password);
        if (!empty($domain_name) && !empty($hub) && !empty($hash) && !empty($password)) {
            return 'OK' . $response['redirectUrl'] = 'https://' . $hub . '?action=autologin&domain=' . $domain_name . '&timestamp=' . $timestamp . '&hash=' . $hash . '&password=' . $password;
        } else {
            return false;
        }
    }

    public function Unsuspend($domain_name)
    {
        $endpoint = 'active';
        $apikey = sha1(trim($this->apikey));
        $timestamp = str_shuffle(time());
        $hash = sha1($apikey . $domain_name . $timestamp);
        $data = [
            'domain' => $domain_name,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];
        if ($data !== array_filter($data, 'strlen') || empty($this->apikey) || empty($this->server)) {
            return json_encode(array(
                'status' => "error",
                'message' =>  'Request missing data'
            ));
        }
        $response = json_decode($this->call($endpoint,  'POST', $data));
        if (is_object($response) && substr($response->status, 0, 2) != 'OK') {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response->status
            ));
        } elseif (is_object($response) && substr($response->status, 0, 2) == 'OK') {
            return json_encode(array(
                'status' => "successful",
                'message' =>  $response->status
            ));
        } else {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response
            ));
        }
    }


    public function Update($domain_name, $domain_package, $domain_status = 'active')
    {
        $apikey = sha1(trim($this->apikey));
        $timestamp = str_shuffle(time());
        $endpoint = 'packageupdate';
        $hash = sha1($apikey . $domain_name . $timestamp);
        $data = [
            'status' => $domain_status,
            'package' => $domain_package,
            'domain' => $domain_name,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];
        if ($data !== array_filter($data, 'strlen') || empty($this->apikey) || empty($this->server)) {
            return json_encode(array(
                'status' => "error",
                'message' =>  'Request missing data'
            ));
        }
        $response = json_decode($this->call($endpoint,  'POST', $data));
        if (is_object($response) && substr($response->status, 0, 2) != 'OK') {
            if(isset($response->token)){
                return json_encode(array(
                    'status' => "error",
                    'message' =>  $response->status,
                    'token' =>  $response->token
                ));
            }else{
                return json_encode(array(
                    'status' => "error",
                    'message' =>  $response->status,
                ));
            }
            
        } elseif (is_object($response) && substr($response->status, 0, 2) == 'OK') {
            return json_encode(array(
                'status' => "successful",
                'message' =>  $response->status,
                'token' =>  $response->token
            ));
        } else {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response
            ));
        }

        return false;
    }

    public function ResetPass($domain_name, $domain_password)
    {

        $apikey = sha1(trim($this->apikey));
        $timestamp = str_shuffle(time());
        $hash = sha1($apikey . $domain_name . $timestamp);

        $password = $this->gpfPassEncrypt($this->apikey, $domain_password);
        $endpoint = 'passwordreset';
        $data = [
            'domain' => $domain_name,
            'password' => $password,
            'hash' => $hash,
            'timestamp' => $timestamp,
        ];
        if ($data !== array_filter($data, 'strlen') || empty($this->apikey) || empty($this->server) ) {
            return json_encode(array(
                'status' => "error",
                'message' =>  'Request missing data'
            ));
        }
        $response = json_decode($this->call($endpoint,  'POST', $data));
        if (is_object($response) && substr($response->status, 0, 2) != 'OK') {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response->status
            ));
        } elseif (is_object($response) && substr($response->status, 0, 2) == 'OK') {
            return json_encode(array(
                'status' => "successful",
                'message' =>  $response->status
            ));
        } else {
            return json_encode(array(
                'status' => "error",
                'message' =>  $response
            ));
        }

        return false;
    }
}
